@@ -7,6 +7,7 @@ applications, such as a simple
 
 import torch
 
+
 class Model(torch.nn.Module):
     """A placeholder model that lists all methods used by the losses defined
     in :mod:`paradime.loss`.
@@ -24,30 +25,28 @@ class Model(torch.nn.Module):
     def classify(self, X: torch.Tensor) -> torch.Tensor:
         raise NotImplementedError()
 
+
 class FullyConnectedEmbeddingModel(Model):
     """A fully connected network for embeddings.
-    
+
     Args:
         in_dim: Input dimension.
         out_dim: Output dimensions.
         hidden_dims: List of hidden layer dimensions.
     """
-    def __init__(self,
-        in_dim: int,
-        out_dim: int,
-        hidden_dims: list[int]
-    ):
+
+    def __init__(self, in_dim: int, out_dim: int, hidden_dims: list[int]):
         super().__init__()
-        
+
         self.layers = torch.nn.ModuleList()
-        
+
         cur_dim = in_dim
         for hdim in hidden_dims:
             self.layers.append(torch.nn.Linear(cur_dim, hdim))
             cur_dim = hdim
         self.layers.append(torch.nn.Linear(cur_dim, out_dim))
 
-        self.alpha = torch.nn.Parameter(torch.tensor(1.))
+        self.alpha = torch.nn.Parameter(torch.tensor(1.0))
 
     def forward(self, x):
         for layer in self.layers[:-1]:
