@@ -12,14 +12,15 @@ import torch
 
 from paradime.types import TensorLike
 
+
 def to_numpy(X: Union[TensorLike, list[float]]) -> np.ndarray:
-    """Converts a tensor-like object to a NumPy array.
-    
+    """Converts a tensor-like object to a numpy array.
+
     Args:
         X: The tensor-like object to be converted.
-    
+
     Returns:
-        The resulting Numpy array.
+        The resulting numpy array.
     """
     if isinstance(X, torch.Tensor):
         return X.detach().cpu().numpy()
@@ -31,27 +32,35 @@ def to_numpy(X: Union[TensorLike, list[float]]) -> np.ndarray:
         raise TypeError(f"Input type {type(X)} not supported")
 
 
-
 def to_torch(X: Union[TensorLike, list[float]]) -> torch.Tensor:
     """Converts a tensor-like object to a PyTorch tensor.
-    
+
     Args:
         X: The tensor-like object to be converted.
-    
+
     Returns:
-        The resulting PyTorch tensor.
+        The resulting PyTorch tensor. If the input was not a PyTorch tensor
+        already, the output tensor will be of type float32 for float inputs and
+        of type int32 for integer inputs.
     """
     if isinstance(X, torch.Tensor):
         return X
     elif isinstance(X, (np.ndarray, list)):
-        return torch.tensor(X, dtype=torch.float)
+        tensor = torch.tensor(X)
+        if tensor.is_floating_point():
+            return tensor.float()
+        elif not tensor.is_complex():
+            return tensor.int()
+        else:
+            return tensor
     else:
         raise TypeError(f"Input type {type(X)} not supported")
 
+
 @functools.cache
 def rowcol_to_triu_index(i: int, j: int, dim: int) -> int:
     """Converts matrix indices to upper-triangular form.
-    
+
     Converts a pair of row and column indices of a symmetrical square array to
     the corresponding index of the list of upper triangular values.
 
@@ -77,6 +86,7 @@ def rowcol_to_triu_index(i: int, j: int, dim: int) -> int:
             "upper-triangular form."
         )
 
+
 @functools.cache
 def triu_to_square_dim(len_triu: int) -> int:
     """Calculates the size of a square matrix given the length of the list of
