@@ -11,12 +11,13 @@ import numpy as np
 from packaging import version
 import torch
 
-def seed_all(seed:int) -> torch.Generator:
+
+def seed_all(seed: int) -> torch.Generator:
     """Sets several seeds to maximize reproducibility.
 
     For infos on reproducibility in PyTorch, see
     https://pytorch.org/docs/stable/notes/randomness.html.
-    
+
     Args:
         seed: The integer to use as a seed.
 
@@ -24,7 +25,7 @@ def seed_all(seed:int) -> torch.Generator:
         The :class:`torch.Generator` instance returned by
         :func:`torch.manual_seed`.
     """
-    os.environ['PYTHONHASHSEED']=str(seed)
+    os.environ["PYTHONHASHSEED"] = str(seed)
     if version.parse(torch.version.cuda) >= version.parse("10.2"):
         os.environ["CUBLAS_WORKSPACE_CONFIG"] = ":4096:8"
     random.seed(seed)
